import sys
import numpy as np

############################ To run this script: ######################################
#
# python3 GenerateData.py NumMachines numJobs seedStart distributionType destinationFolderPath stdDev
#
# First argument (NumMachines) refers to the total number of identical machines in simulation
#
# Second argument (numJobs) refers to the total number of jobs in simulation
#
# Third argument (seedStart=0) is the starting seed
#
# Forth argument defines the distribution type to be used to generate job processing times
# 
# Fifth argument is the folder name to place the generated files in 
# 
# Sixth argument is the standard deviation to generate data (larger standard deviations 
# leads to higher variance between job processing times) - only used for lognormal distribution
#
# Seventh argument is the mean value to generate data - only used for lognormal distribution
#
######################################################################################

NumMachines = int(sys.argv[1])
NumJobs = int(sys.argv[2])
seedStart = int(sys.argv[3])
distributionType = str(sys.argv[4])
destinationFolderPath = str(sys.argv[5])
stdDev = float(sys.argv[6])
mean = float(sys.argv[7])

# Set the random generator to a fix start value for the reproducibility of experiments
np.random.seed(seedStart)

# generate the execution of jobs with desired distribution
if distributionType == "uniform":
    p_i = np.random.uniform(size = NumJobs, low = 1, high = 218) 

if distributionType == "lognormal":
    p_i = np.random.lognormal(mean, stdDev, NumJobs)

# sort execution times in descending order
# TODO: This sorting should be in engine.py, the generated data is read
p_i[::-1].sort()

# Set the title and its explanations for the input file for machines 
filename = destinationFolderPath + '/Jobs_NumJobs' + str(NumJobs) + '_'+ distributionType 
filename += '_seed' + str(seedStart) + '_stdDev' + str(stdDev) + '.swf'
with open(filename, 'w') as f:
    f.write(';' + 'This file is generated by GenerateData.py' + '\n')
    f.write(';' + 'Each line represents the corresponding data of one job only' + '\n')
    f.write(';' + 'Field names and their sizes are as follows:' + '\n')
    f.write(';' + 'jobId_i[1] + processingTime[1]' + '\n')
    f.write(';' + '\n')

# Write the given and random data into the file
with open(filename, 'a') as f:
    for i in range(NumJobs):
        # Write all the values into the file
        line = str(i) + ' ' + str(p_i[i])  + '\n' 
        f.write( line )
