# Files and Folders

### 1. `DataPaper/`

- By default, this folder consists of all the files of randomly generated job processing times that we used in our experiments

- **Warning** When the `generate_data_files.sh` script is executed, these files will be overwritten

### 2. `ResultsPaper/`

- By default, this folder consists of all the result files of our experiments

- **Warning** When the `run_in_parallel.sh` script is executed, these files will be overwritten

### 3. `simulator/`

- This folder consists of all simulation codes written in python (It should not be necessary to change the file in this folder)

### 4. `test/`

- There is only `test.py` inside of this folder. This file takes the user given arguments and calls the simulator engine for each method 

- It should not be necessary to change `test.py`. Instead, the arguments for this file can be changed within `run_in_parallel.sh` script which invokes `test.py`

### 5. `generate_data_files.sh`

- This file should be executed as `$source generate_data_files.sh` to generate the random processing times of the jobs within `DataPaper/` folder.

- By default, this file generates exactly the same data as we used in the paper (it takes approximately 1 min to produce all data).

- In case different random data is to be generated, you have to change the corresponding arguments inside of this script (all arguments are explained in the beginning of the script).

### 6. `GenerateData.py`

- This file is invoked by `generate_data_files.sh` execution with the given arguments to generate the corresponding random data.

- You should not modify this file

### 7. `run_in_parallel.sh`

- This file should be executed as `$source run_in_parallel.sh` to generate the experimental results within `ResultsPaper/` folder.

- By default, this file generates exactly the same results as we used in the paper (it takes long time to produce all results, so be sure to reduce the number of arguments before the execution).

- In case different results need to be generated, you have to change the corresponding arguments inside of this script (all arguments are explained in the beginning of the script).

- **Warning** If the corresponding random data does not exist in the `DataPaper/` folder, it cannot produce results.

### 8. `BoxPlotSchWithRejection_paper.R`

- This file should be executed as `Rscript --vanilla BoxPlotSchWithRejection_paper.R` to produce the figures.

- By default, this file generates exactly the same results as the figure with "40 and 80 jobs" in the paper.

- In case, you want to generate exactly the same results as the figure with "200 and 400 jobs" in the paper, you need to set `numJobs = c(200, 400)` and `ylim = 1.0185` on the lines 17 and 18 respectively.

- In case different results need to be generated, you have to change the corresponding arguments inside of this script (all arguments are explained in the beginning of the script).

- **Warning** If the corresponding files do not exist in the `DataPaper/` and `ResultsPaper/` folders, it cannot produce the figures. But, by default, all the needed data is inside the folders to produce the paper figures

### 9. `StdDevVSCost_SchWithRejection_numMachine20_eps0.05_FEWERJOBS.pdf`

- The figure with "40 and 80 jobs" in the paper.

### 10. `StdDevVSCost_SchWithRejection_numMachine20_eps0.05_MOREJOBS.pdf`

- The figure with "200 and 400 jobs" in the paper.

### 11. `README.md`

- The current file which explains all the important information on the current directory

# Important Note 

- We used Gurobi solver to generate our lower bound results. In this `README.md` file, we explain how to reproduce the experimental results, both when 1) Gurobi solver installed 2) and not (as well as links for installing Gurobi). 

- In case Gurobi is not installed, you will reproduce the figures with "costs wrt W (sum of the processing times of all jobs)" instead of "relative costs with respect to the lower bound".

- **Important** Academic people can use Gurobi for free; else it is commercial.

- For academical Gurobi installation, you should sign up to Gurobi's website with an academic email. Then, the licence is directly in the account (it is straightforward).

- This registration can be done by following the instructions on the website: https://www.gurobi.com/academia/academic-program-and-licenses/

- Then, the guide to install Gurobi can be found in: https://www.youtube.com/watch?v=yNmeG6Wom1o&ab_channel=GurobiOptimization

- For installing the Python interface to Gurobi, the commandline is `$python3 -m pip install gurobipy`

# Producing Figures for the Paper

## STEP 1: Produce Random Processing Times

Execute the `generate_data_files.sh` script as `$source generate_data_files.sh` to generate the random processing times of the jobs:

- Be sure to be in the same directory as the script to execute `generate_data_files.sh` script.
- This execution will create the files in `DataPaper` containing all the corresponding data files with a possibility to overwrite the existing files.
- The script is configured to produce the exactly same results as presented in the paper. No modifications are needed.
- This operation can take up to 1 minutes.
- To generate different job sets with different processing times, adjust the variables inside `generate_data_files.sh`.
- The functionality of all 7 variables is explained at the beginning of the script. Only modify these variables to generate different random data.

## STEP 2: Run the Experiments

Execute the `run_in_parallel.sh` script as `$source run_in_parallel.sh` to run the experiments:

- Be sure to be in the same directory as the script to execute `run_in_parallel.sh` script
- **Important**: Ensure Gurobi solver is installed for obtaining experimental results. If Gurobi is not installed, set `withLB=0` on line 35 inside `run_in_parallel.sh`.
- Gurobi solver is used solely for computing the lower bound (LB); setting `withLB=0` avoids producing lower bound results.
- This execution will create the results inside `ResultsPaper` containing all the corresponding experimental result files.
- The script is configured to produce the exact results as presented in the paper. No modifications are needed.
- This operation can take too long time, thus be sure to give only one value for each argument before running it to get the results only in seconds.
- To generate different experimental results, firstly ensure that the required data is generated in the previous step (by changing `generate_data_files.sh` script), and then adjust the variables inside `run_in_parallel.sh` accordingly.
- The functionality of all 11 variables is explained at the beginning of the script. Modify only these variables for generating different experimental results.

## STEP 3: Plot the Figures

Run the `BoxPlotSchWithRejection_paper.R` as `$Rscript --vanilla BoxPlotSchWithRejection_paper.R` to plot the figures:

- **Important**: Ensure Gurobi solver is installed for obtaining the figure. If Gurobi is not installed, set `withLB=FALSE` on line 14 within this script.
- `withLB=FALSE` ensures the figures to display the "costs wrt W (sum of the processing times of all jobs)" instead of "relative costs with respect to the lower bound".
- This script generate the figures (in pdf format in the current directory) for "40 and 80" jobs by default. 
- If the figure illustrating the "200 and 400" jobs case is to be generated, set `numJobs=c(200,400)` and `ylim=1.0185`on lines 17 and 18, respectively.
- All the 12 parameters that might be changed for different experimental results are specified in the beginning of the script. It should not be necessary to change another thing in this script.
