#!/bin/bash

########## Explanation of this script: ###############
#
# This script is for running GenerateData.py with different arguments. 
# The arguments can be changed within this script to generate different random data
#
########## Meanings of the arguments: ###############
#
# dataFolderPath: refers to the folder where to palce the generated data files (default: "DataPaper" folder 
# will be created in the current folder). 
#
# numMach: refers to the total number of identical machines in simulation (default: 20)
#
# mean: refers to the mean value used in the lognormal function to generate random processing times (default: 3)
#
# Ratio: is used to compute the number of jobs in the simulation (numJob). That is, "numJob = Ratio * numMach" (default: 2 4 10 20)
#
# seed: is the seed used to set the random generator for reproducibility (default: we used 30 experiments with seeds 0,1,2,...,29)
#
# dist: is the name of the distribution (default: lognormal)
# 
# stdDev: is the standard deviation used to generate processing times with lognormal distribution (default: 0.5 0.7 1.0)  
# 
#####################################################

dataFolderPath="DataPaper"
numMach=20
mean=3

mkdir -p "$dataFolderPath"

for Ratio in 2 4 10 20
do
	numJob=$((Ratio * numMach))
	for seed in {0..29}
	do 
		for dist in "lognormal" 
		do
			for stdDev in 0.5 0.7 1.0 
			do
				python3 GenerateData.py $numMach $numJob $seed $dist $dataFolderPath $stdDev $mean
			done
		done
	done
done

