#!/usr/bin/env bash

########## Explanations of this script: ###############
#
# This script is for running test.py in with different arguments
#
########## Meanings of the arguments: ###############
#
# debug: If the debug mode is on (default: 0) 
#
# withLB: If the lower bound results should be obtained or not (default: 1, in case Gurobi is not installed, please set withLB to 0)
#
# eps: The precision coeffient is "1+eps" when computing the makespan in BEKP algo. (default: 0.05)
#
# dataFolder: THe folder where the generated random data is placed (default: "DataPaper")
#
# resultsFolder: The folder where the experimental results will be placed (default: "ResultsPaper")
#
# dist: is the name of the distribution (default: lognormal)
#
# numMach: refers to the total number of identical machines in simulation (default: 20)
#
# ratioJobsMachines: is used to compute the number of jobs in the simulation (numJob). 
# That is, "numJob = ratioJobsMachines * numMach" (default: 2 4 10 20)
#
# seed: is the seed used to set the random generator for reproducibility (default: we used 30 experiments with seeds 0,1,2,...,29)
#
# rho: The rejection coefficient (default: 1.5 4.0)
#
# stdDev: is the standard deviation used to generate processing times with lognormal distribution (default: 0.5 0.7 1.0)  
# 
#####################################################

debug=0
withLB=1
eps=0.05
dataFolder="DataPaper"
resultsFolder="ResultsPaper"
dist="lognormal"

mkdir -p $resultsFolder

for numMach in 20
do
	for ratioJobsMachines in 2 4 10 20
	do
		for seed in {0..29} 
		do
			for rho in 1.5 4.0
			do
				numJob=$((ratioJobsMachines * numMach))
				for stdDev in "0.5" "0.7" "1.0"
				do
					#echo $numMach $numJob $dist $dataFolder $resultsFolder $rho $eps $debug $seed $stdDev $withLB
					python3 test/test.py $numMach $numJob $dist $dataFolder $resultsFolder $rho $eps $debug $seed $stdDev $withLB
				done
			done
		done
	done
done
