class Solution:
    """Contains information about a solution.

    Attributes
    ----------
    newdecision : bool
        If scheduling will be applied or not

    assignedMachineIDs : Int list
        Keeps the assigned "node_id"s of the corresponding jobs. 
        Default: -2, meaning that the corresponding job is rejected

    jobStartTimes : Int list
        Keeps the "starting time"s of the corresponding jobs. 
        Default: 0

    machineStartTimes : Int list
        Keeps the soonest availabilities of the corresponding nodes
        Default: 0

    Notes
    -----

    """
    def __init__(self, m, n):
        self.newdecision = True
        self.assignedMachineIDs = [-2] * n
        self.jobStartTimes = [0] * n
        self.machineStartTimes = [0] * m
        self.extra_info = -2

    def copy(self):
        new_solution = Solution(len(self.machineStartTimes), len(self.assignedMachineIDs))
        new_solution.newdecision = self.newdecision
        new_solution.assignedMachineIDs = self.assignedMachineIDs.copy()
        new_solution.jobStartTimes = self.jobStartTimes.copy()
        new_solution.machineStartTimes = self.machineStartTimes.copy()
        return new_solution

    def getNewDecision(self):
        return self.newdecision

    def getX(self):
        return self.assignedMachineIDs
    
    def getCompletionTimeOfJob(self, job):
        return self.jobStartTimes[job.index] + job.run
    
    def getJobStartTimes(self):
        return self.jobStartTimes
    
    def getMachineStartTimes(self):
        return self.machineStartTimes
    
    def getArea(self):
        return sum(self.machineStartTimes)
    
    def getMakespan(self):
        return max(self.machineStartTimes)
    
    def getCost(self, W, m, rho):
        return self.getMakespan() * m + rho * (W-self.getArea())

    def assignJob(self, job, machine_number):
        self.assignedMachineIDs[job.index] = machine_number  
        self.jobStartTimes[job.index] = self.machineStartTimes[machine_number] 
        self.machineStartTimes[machine_number] += job.run

    def assignSeveralJobsToOneMachine(self, list_of_jobs, machine_number):
        # TODO: Right now we assign the jobs to the same machine in any order
        for job in list_of_jobs: 
            self.assignJob(job, machine_number)

    # This function assigns the jobs with LPT actually
    def assignGreedy(self, set, C_fit):
        # Sort jobs in descending order of processing times
        if set.jobs:
            sortedJobs = sorted(set.jobs, key=lambda x: x.run, reverse=True)

            for job in sortedJobs:
                # Find the machine on which the job can start the soonest
                minMachine = min(enumerate(self.machineStartTimes), key=lambda x: x[1])[0]

                if self.machineStartTimes[minMachine] + job.run <= C_fit:
                    self.assignJob(job, minMachine)

    def __str__(self):
        """
        Function used when we try to print() an object of this
        class.
        """
        text = ('{' +
                f'X_j: {self.getX()} ' + f'B_j: {self.getJobStartTimes()} ' + 
                  f'makespan_3: {self.getMakespan()} ' +  
                  f'area: {self.getArea()} ' + 
                  f'machineStartTimes: {self.getMachineStartTimes()} ' +
                '}')
        return text
