#######################################################
# In order to execute:
#
# python3 test.py NumMachines NumJobs distribution 
# dataFolder resultFolder rho epsilon debug seed stdDev
#
#######################################################
import sys
from itertools import combinations
import time

# Add the parent directory to the path so we can import
# code from our simulator
sys.path.append('./')

from simulator.engine import Engine              

if __name__ == '__main__':

        numMachines = int(sys.argv[1]) 
        numJobs = int(sys.argv[2])
        distribution = str(sys.argv[3])
        dataFolder = str(sys.argv[4])
        resultFolder = str(sys.argv[5])  
        rho = float(sys.argv[6])  
        epsilon = float(sys.argv[7])
        debug = int(sys.argv[8])
        seed = str(sys.argv[9])
        stdDev = str(sys.argv[10])
        withLB = int(sys.argv[11])

        print( 'Running the experiments with ' + str(numMachines) 
              + ' identical machines, ' + str(numJobs) + ' jobs and for ' + 
              distribution + ' distribution', ' rho: ', rho )

        start = time.time()
        simulator = Engine('BEKP', numMachines, numJobs, 
                    seed, stdDev, dataFolder, distribution, debug)
        end = time.time()
        start_BEKP = time.time()
        makespan = simulator.run(seed, stdDev, rho, epsilon, resultFolder, distribution)
        end_BEKP = time.time()

        simulator = Engine('LPT', numMachines, numJobs,
                    seed, stdDev, dataFolder, distribution, debug)
        start_LPT = time.time()
        makespan = simulator.run(seed, stdDev, rho, epsilon, resultFolder, distribution)
        end_LPT = time.time()

        simulator = Engine('LIULU', numMachines, numJobs, 
                    seed, stdDev, dataFolder, distribution, debug)
        start_LIULU = time.time()
        makespan = simulator.run(seed, stdDev, rho, epsilon, resultFolder, distribution)
        end_LIULU = time.time() 

        if withLB:              
                simulator = Engine('LB', numMachines, numJobs, 
                    seed, stdDev, dataFolder, distribution, debug)
                start_LB = time.time()
                makespan = simulator.run(seed, stdDev, rho, epsilon, resultFolder, distribution)
                end_LB = time.time()
           
